package main

import (
	"os"
	"fmt"
	"time"
	"strconv"
	"math/rand" 
)

func printMineMap(mineMap [][]rune) {
	for _,row := range(mineMap) {
		for _,cell := range(row) {
			switch cell {
				case '*','1','2','3','4','5','6','7','8':
					fmt.Printf("%c ",cell)
				default:
					fmt.Print("◻ ")
			}
		}
		fmt.Println()
	}
}

func main() {
	if len(os.Args) != 4 {
		fmt.Println("syntax: mine <Height> <Width> <Mines>")
		return
	}
	
	//define parameters
	height, _ := strconv.Atoi(os.Args[1])
	width, _ := strconv.Atoi(os.Args[2])
	mines, _ := strconv.Atoi(os.Args[3])
	
	//initialize rand
	rand.Seed(time.Now().UnixNano())
	
	//initialize map
	var mineMap [][]rune = make([][]rune, height) //initialize height
	var exploredMap [][]bool = make([][]bool, height) //initialize height
	for i := 0; i < height; i++ {
		mineMap[i] = make([]rune, width) //initialize width
		exploredMap[i] = make([]bool, width) //initialize width
	}
	
	//insert bombs 
	var shuffledPositions []int = rand.Perm(height*width) //create a slice with height*width elements and shuffle it.
	//fmt.Println(shuffledPositions)
	
	for i:=0; i<mines; i++ { //pick first n elements as bombs and insert it in the map
		var sequentialPosition int = shuffledPositions[i] //get the sequential position of the first bomb
		var verticalPosition int = sequentialPosition/width
		var horizontalPosition int = sequentialPosition%width

		//fmt.Println("seq:",sequentialPosition,"vert:",verticalPosition,"hor:",horizontalPosition)
		mineMap[verticalPosition][horizontalPosition] = '*'
	}

	printMineMap(mineMap)
}
