package main

import (
	"os"
	"fmt"
	"time"
	"strconv"
	"math/rand" 
)

var exploredCount int = 0

func printMineMap(mineMap [][]rune) {
	for _,row := range(mineMap) {
		for _,cell := range(row) {
			switch cell {
				case '*','1','2','3','4','5','6','7','8':
					fmt.Printf("%c ",cell)
				default:
					fmt.Print("◻ ")
			}
		}
		fmt.Println("|")
	}
}

func printExploredMap(mineMap [][]rune, exploredMap [][]bool) {
	for i,row := range(mineMap) {
		for j,cell := range(row) {
			if exploredMap[i][j] == true {
				switch cell {
					case '*','1','2','3','4','5','6','7','8':
						fmt.Printf("%c ",cell)
					default:
						fmt.Print("◻ ")
				}
			} else {
				fmt.Print("◼ ")
			}
		}
		fmt.Println("|")
	}
}

func floodFill(mineMap [][]rune, exploredMap *[][]bool, i int, j int) {
	switch {
		case i<0, j<0, i>=len(mineMap), j>=len(mineMap[0]), (*exploredMap)[i][j] == true:
			return
	}

	switch mineMap[i][j] {
		case '*','1','2','3','4','5','6','7','8':
			return
		default:
			(*exploredMap)[i][j] = true
			exploredCount++
			floodFill(mineMap, exploredMap, i+1, j)
			floodFill(mineMap, exploredMap, i-1, j)
			floodFill(mineMap, exploredMap, i, j+1)
			floodFill(mineMap, exploredMap, i, j-1)
	}
}

func main() {
	if len(os.Args) != 4 {
		fmt.Println("syntax: mine <Height> <Width> <Mines>")
		return
	}
	
	//define parameters
	height, _ := strconv.Atoi(os.Args[1])
	width, _ := strconv.Atoi(os.Args[2])
	mines, _ := strconv.Atoi(os.Args[3])
	
	//initialize rand
	rand.Seed(time.Now().UnixNano())
	
	//initialize map
	var mineMap [][]rune = make([][]rune, height) //initialize height
	var exploredMap [][]bool = make([][]bool, height) //initialize height
	for i := 0; i < height; i++ {
		mineMap[i] = make([]rune, width) //initialize width
		exploredMap[i] = make([]bool, width) //initialize width
	}
	
	//insert bombs 
	var shuffledPositions []int = rand.Perm(height*width) //create a slice with height*width elements and shuffle it.
	//fmt.Println(shuffledPositions)
	
	for i:=0; i<mines; i++ { //pick first n elements as bombs and insert it in the map
		var sequentialPosition int = shuffledPositions[i] //get the sequential position of the first bomb
		var verticalPosition int = sequentialPosition/width
		var horizontalPosition int = sequentialPosition%width

		//fmt.Println("seq:",sequentialPosition,"vert:",verticalPosition,"hor:",horizontalPosition)
		mineMap[verticalPosition][horizontalPosition] = '*'
		
		var minH, maxH, minV, maxV int
		
		//add markers
		if verticalPosition == 0 { //we cannot place the marker above
			minV = 0
		} else {
			minV = verticalPosition-1
		}

		if verticalPosition == height-1 { //we cannot place the marker under
			maxV = height
		} else {
			maxV = verticalPosition+2
		}

		if horizontalPosition == 0 { //we cannot place the marker above
			minH = 0
		} else {
			minH = horizontalPosition-1
		}

		if horizontalPosition == width-1 { //we cannot place the marker under
			maxH = width
		} else {
			maxH = horizontalPosition+2
		}

		for i:=minV; i<maxV; i++ {
			for j:=minH; j<maxH; j++ {
				switch {
					case mineMap[i][j] == 0:
						mineMap[i][j] = '1'
					case mineMap[i][j] > '0' &&  mineMap[i][j] < '8':
						mineMap[i][j]++
				}
			}
		}
	}
	
	for {
		printExploredMap(mineMap, exploredMap)
		fmt.Print("Explore a point (x,y): ")
		var x,y int
		fmt.Scanf("%d %d\n", &x, &y)
		
		if exploredMap[y][x] == true {
			fmt.Println("already explored that smartypants")
		} else {
			switch mineMap[y][x] {
				case '*':
					fmt.Println("BOOM, you lose")
					printMineMap(mineMap)
					return
				case '1','2','3','4','5','6','7','8':
					exploredMap[y][x] = true
					exploredCount++
				default:
					floodFill(mineMap, &exploredMap, y, x)
			}
		}
		
		if exploredCount == height*width-mines {
			fmt.Println("You won!")
			printMineMap(mineMap)
			return
		}
	}
}
